@@ -5,84 +5,144 @@ use std::mem;
 use std::iter;
 use std::io::prelude::*;
 
-use traits::{Parameter, SetParameter};
-use common::Frame;
-use util;
+use crate::common::{Block, Frame};
 
 mod decoder;
 pub use self::decoder::{
-    PLTE_CHANNELS, StreamingDecoder, Decoded, DecodingError, Extensions
+    PLTE_CHANNELS, StreamingDecoder, Decoded, DecodingError, DecodingFormatError, Extensions,
+    Version
 };
 
 const N_CHANNELS: usize = 4;
 
-impl<T, R> Parameter<Decoder<R>> for T
-where T: Parameter<StreamingDecoder>, R: Read {
-    type Result = ();
-    fn set_param(self, this: &mut Decoder<R>) {
-        this.decoder.set(self);
-    }
-
-}
-
 /// Output mode for the image data
-#[derive(PartialEq, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq)]
 #[repr(u8)]
 pub enum ColorOutput {
     /// The decoder expands the image data to 32bit RGBA.
     /// This affects:
     ///
-    ///  - The buffer buffer of the `Frame` returned by `Reader::read_next_frame`.
-    ///  - `Reader::fill_buffer`, `Reader::buffer_size` and `Reader::line_length`.
+    ///  - The buffer buffer of the `Frame` returned by `Decoder::read_next_frame`.
+    ///  - `Decoder::fill_buffer`, `Decoder::buffer_size` and `Decoder::line_length`.
     RGBA = 0,
     /// The decoder returns the raw indexed data.
     Indexed = 1,
 }
 
-impl<R: Read> Parameter<Decoder<R>> for ColorOutput {
-    type Result = ();
-    fn set_param(self, this: &mut Decoder<R>) {
-        this.color_output = self
-    }
-}
-
-#[derive(Debug)]
-/// Memory limit in bytes. `MemoryLimit::Some(0)` means
+#[derive(Clone, Debug)]
+/// Memory limit in bytes. `MemoryLimit(0)` means
 /// that there is no memory limit set.
 pub struct MemoryLimit(pub u32);
 
-impl<R: Read> Parameter<Decoder<R>> for MemoryLimit {
-    type Result = ();
-    fn set_param(self, this: &mut Decoder<R>) {
-        let MemoryLimit(limit) = self;
-        this.memory_limit = limit
-    }
-}
+impl MemoryLimit {
+    /// Enforce no memory limit.
+    ///
+    /// If you intend to process images from unknown origins this is a potentially dangerous
+    /// constant to use, as your program could be vulnerable to decompression bombs. That is,
+    /// malicious images crafted specifically to require an enormous amount of memory to process
+    /// while having a disproportionately small file size.
+    ///
+    /// The risks for modern machines are a bit smaller as the dimensions of each frame can not
+    /// exceed `u32::MAX` (~4Gb) but this is still a significant amount of memory.
+    pub const NONE: MemoryLimit = MemoryLimit(0);
 
-/// GIF decoder
-pub struct Decoder<R: Read> {
-    r: R,
-    decoder: StreamingDecoder,
-    memory_limit: u32,
-    color_output: ColorOutput,
-}
+    fn buffer_size(&self, color: ColorOutput, width: u16, height: u16) -> Option<usize> {
+        let pixels = u32::from(width) * u32::from(height);
 
-impl<R: Read> Decoder<R> {
-    /// Creates a new decoder builder
-    pub fn new(r: R) -> Decoder<R> {
-        Decoder {
-            r: r,
-            decoder: StreamingDecoder::new(),
-            memory_limit: 50_000_000, // 50 MB
-            color_output: ColorOutput::Indexed
+        let bytes_per_pixel = match color {
+            ColorOutput::Indexed => 1,
+            ColorOutput::RGBA => 4,
+        };
+
+        if self.0 > 0 && pixels > self.0 / bytes_per_pixel {
+            None
+        } else {
+            Some(pixels as usize * bytes_per_pixel as usize)
         }
     }
-    
+}
+
+/// Options for opening a GIF decoder.
+#[derive(Clone, Debug)]
+pub struct DecodeOptions {
+    memory_limit: MemoryLimit,
+    color_output: ColorOutput,
+    check_frame_consistency: bool,
+    check_for_end_code: bool,
+    allow_unknown_blocks: bool,
+}
+
+impl DecodeOptions {
+    /// Creates a new decoder builder
+    pub fn new() -> DecodeOptions {
+        DecodeOptions {
+            memory_limit: MemoryLimit(50_000_000), // 50 MB
+            color_output: ColorOutput::Indexed,
+            check_frame_consistency: false,
+            check_for_end_code: false,
+            allow_unknown_blocks: false,
+        }
+    }
+
+    /// Configure how color data is decoded.
+    pub fn set_color_output(&mut self, color: ColorOutput) {
+        self.color_output = color;
+    }
+
+    /// Configure a memory limit for decoding.
+    pub fn set_memory_limit(&mut self, limit: MemoryLimit) {
+        self.memory_limit = limit;
+    }
+
+    /// Configure if frames must be within the screen descriptor.
+    ///
+    /// The default is `false`.
+    ///
+    /// When turned on, all frame descriptors being read must fit within the screen descriptor or
+    /// otherwise an error is returned and the stream left in an unspecified state.
+    ///
+    /// When turned off, frames may be arbitrarily larger or offset in relation to the screen. Many
+    /// other decoder libraries handle this in highly divergent ways. This moves all checks to the
+    /// caller, for example to emulate a specific style.
+    pub fn check_frame_consistency(&mut self, check: bool) {
+        self.check_frame_consistency = check;
+    }
+
+    /// Configure if LZW encoded blocks must end with a marker end code.
+    ///
+    /// The default is `false`.
+    ///
+    /// When turned on, all image data blocks—which are LZW encoded—must contain a special bit
+    /// sequence signalling the end of the data. LZW processing terminates when this code is
+    /// encountered. The specification states that it must be the last code output by the encoder
+    /// for an image.
+    ///
+    /// When turned off then image data blocks can simply end. Note that this might silently ignore
+    /// some bits of the last or second to last byte.
+    pub fn check_lzw_end_code(&mut self, check: bool) {
+        self.check_for_end_code = check;
+    }
+
+    /// Configure if unknown blocks are allowed to be decoded.
+    ///
+    /// The default is `false`.
+    ///
+    /// When turned on, the decoder will allow unknown blocks to be in the
+    /// `BlockStart` position.
+    ///
+    /// When turned off, decoded block starts must mark an `Image`, `Extension`,
+    /// or `Trailer` block. Otherwise, the decoded image will return an error.
+    /// If an unknown block error is returned from decoding, enabling this
+    /// setting may allow for a further state of decoding on the next attempt.
+    pub fn allow_unknown_blocks(&mut self, check: bool) {
+        self.allow_unknown_blocks = check;
+    }
+
     /// Reads the logical screen descriptor including the global color palette
     ///
-    /// Returns a `Reader`. All decoder configuration has to be done beforehand.
-    pub fn read_info(self) -> Result<Reader<R>, DecodingError> {
-        Reader::new(self.r, self.decoder, self.color_output, self.memory_limit).init()
+    /// Returns a `Decoder`. All decoder configuration has to be done beforehand.
+    pub fn read_info<R: Read>(self, r: R) -> Result<Decoder<R>, DecodingError> {
+        Decoder::with_no_init(r, StreamingDecoder::with_options(&self), self).init()
     }
 }
 
@@ -98,7 +158,7 @@ impl<R: Read> ReadDecoder<R> {
             let (consumed, result) = {
                 let buf = self.reader.fill_buf()?;
                 if buf.len() == 0 {
-                    return Err(DecodingError::Format(
+                    return Err(DecodingError::format(
                         "unexpected EOF"
                     ))
                 }
@@ -107,7 +167,7 @@ impl<R: Read> ReadDecoder<R> {
             self.reader.consume(consumed);
             match result {
                 Decoded::Nothing => (),
-                Decoded::BlockStart(::common::Block::Trailer) => {
+                Decoded::BlockStart(Block::Trailer) => {
                     self.at_eof = true
                 },
                 result => return Ok(unsafe{
@@ -122,36 +182,40 @@ impl<R: Read> ReadDecoder<R> {
 
 #[allow(dead_code)]
 /// GIF decoder
-pub struct Reader<R: Read> {
+pub struct Decoder<R: Read> {
     decoder: ReadDecoder<R>,
     color_output: ColorOutput,
-    memory_limit: u32,
+    memory_limit: MemoryLimit,
     bg_color: Option<u8>,
     global_palette: Option<Vec<u8>>,
     current_frame: Frame<'static>,
     buffer: Vec<u8>,
-    // Offset in current frame
-    offset: usize
-
 }
 
-impl<R> Reader<R> where R: Read {
-    fn new(reader: R, decoder: StreamingDecoder,
-           color_output: ColorOutput, memory_limit: u32
-    ) -> Reader<R> {
-        Reader {
+impl<R> Decoder<R> where R: Read {
+    /// Create a new decoder with default options.
+    pub fn new(reader: R) -> Result<Self, DecodingError> {
+        DecodeOptions::new().read_info(reader)
+    }
+
+    /// Return a builder that allows configuring limits etc.
+    pub fn build() -> DecodeOptions {
+        DecodeOptions::new()
+    }
+
+    fn with_no_init(reader: R, decoder: StreamingDecoder, options: DecodeOptions) -> Decoder<R> {
+        Decoder {
             decoder: ReadDecoder {
                 reader: io::BufReader::new(reader),
-                decoder: decoder,
+                decoder,
                 at_eof: false
             },
             bg_color: None,
             global_palette: None,
             buffer: Vec::with_capacity(32),
-            color_output: color_output,
-            memory_limit: memory_limit,
+            color_output: options.color_output,
+            memory_limit: options.memory_limit,
             current_frame: Frame::default(),
-            offset: 0
         }
     }
     
@@ -174,14 +238,14 @@ impl<R> Reader<R> where R: Read {
                     // palette has been read.
                     unreachable!()
                 },
-                None => return Err(DecodingError::Format(
-                    "File does not contain any image data"
+                None => return Err(DecodingError::format(
+                    "file does not contain any image data"
                 ))
             }
         }
         // If the background color is invalid, ignore it
-        if let &Some(ref palette) = &self.global_palette {
-            if self.bg_color.unwrap_or(0) as usize >= palette.len() {
+        if let Some(ref palette) = self.global_palette {
+            if self.bg_color.unwrap_or(0) as usize >= (palette.len() / PLTE_CHANNELS) {
                 self.bg_color = None;
             }
         }
@@ -195,16 +259,8 @@ impl<R> Reader<R> where R: Read {
                 Some(Decoded::Frame(frame)) => {
                     self.current_frame = frame.clone();
                     if frame.palette.is_none() && self.global_palette.is_none() {
-                        return Err(DecodingError::Format(
-                            "No color table available for current frame."
-                        ))
-                    }
-                    if self.memory_limit > 0  && (
-                        (frame.width as u32 * frame.height as u32)
-                        > self.memory_limit
-                    ) {
-                        return Err(DecodingError::Format(
-                            "Image is too large to decode."
+                        return Err(DecodingError::format(
+                            "no color table available for current frame"
                         ))
                     }
                     break
@@ -222,8 +278,20 @@ impl<R> Reader<R> where R: Read {
     /// Do not call `Self::next_frame_info` beforehand.
     /// Deinterlaces the result.
     pub fn read_next_frame(&mut self) -> Result<Option<&Frame<'static>>, DecodingError> {
-        if self.next_frame_info()?.is_some() {
-            let mut vec = vec![0; self.buffer_size()];
+        if let Some(frame) = self.next_frame_info()? {
+            let (width, height) = (frame.width, frame.height);
+            let pixel_bytes = self.memory_limit
+                .buffer_size(self.color_output, width, height)
+                .ok_or_else(|| {
+                    DecodingError::format("image is too large to decode")
+                })?;
+
+            debug_assert_eq!(
+                pixel_bytes, self.buffer_size(),
+                "Checked computation diverges from required buffer size"
+            );
+
+            let mut vec = vec![0; pixel_bytes];
             self.read_into_buffer(&mut vec)?;
             self.current_frame.buffer = Cow::Owned(vec);
             self.current_frame.interlaced = false;
@@ -244,13 +312,13 @@ impl<R> Reader<R> where R: Read {
             let height = self.current_frame.height as usize;
             for row in (InterlaceIterator { len: height, next: 0, pass: 0 }) {
                 if !self.fill_buffer(&mut buf[row*width..][..width])? {
-                    return Err(DecodingError::Format("Image truncated"))
+                    return Err(DecodingError::format("image truncated"))
                 }
             }
         } else {
             let buf = &mut buf[..self.buffer_size()];
             if !self.fill_buffer(buf)? {
-                return Err(DecodingError::Format("Image truncated"))
+                return Err(DecodingError::format("image truncated"))
             }
         };
         Ok(())
@@ -292,7 +360,7 @@ impl<R> Reader<R> where R: Read {
                     },
                     Indexed => {
                         let len = cmp::min(buf.len(), $data.len());
-                        util::copy_memory(&$data[..len], &mut buf[..len]);
+                        buf[..len].copy_from_slice(&$data[..len]);
                         (len, 1)
                     }
                 }
@@ -301,9 +369,8 @@ impl<R> Reader<R> where R: Read {
         let buf_len = self.buffer.len();
         if buf_len > 0 {
             let (len, channels) = handle_data!(&self.buffer);
-            // This is WRONG!!!! Cuts form the wrong side…
-            self.buffer.truncate(buf_len-len);
-            let buf_ = buf; buf = &mut buf_[len*channels..];
+            let _ = self.buffer.drain(..len);
+            buf = &mut buf[len*channels..];
             if buf.len() == 0 {
                 return Ok(true)
             }
@@ -312,11 +379,11 @@ impl<R> Reader<R> where R: Read {
             match self.decoder.decode_next()? {
                 Some(Decoded::Data(data)) => {
                     let (len, channels) = handle_data!(data);
-                    let buf_ = buf; buf = &mut buf_[len*channels..]; // shorten buf
+                    buf = &mut buf[len*channels..]; // shorten buf
                     if buf.len() > 0 {
                         continue
                     } else if len < data.len() {
-                        self.buffer.extend(data[len..].iter().map(|&v| v));
+                        self.buffer.extend_from_slice(&data[len..]);
                     }
                     return Ok(true)
                 },
@@ -346,8 +413,8 @@ impl<R> Reader<R> where R: Read {
         // TODO prevent planic
         Ok(match self.current_frame.palette {
             Some(ref table) => &*table,
-            None => &*self.global_palette.as_ref().ok_or(DecodingError::Format(
-                "No color table available for current frame."
+            None => &*self.global_palette.as_ref().ok_or(DecodingError::format(
+                "no color table available for current frame"
             ))?,
         })
     }
@@ -402,37 +469,9 @@ mod test {
 
     use super::{Decoder, InterlaceIterator};
     
-    /* Commented because test::Bencher is unstable
-    extern crate test;
-    use std::io::prelude::*;
-    #[bench]
-    fn bench_tiny(b: &mut test::Bencher) {
-        let mut data = Vec::new();
-        File::open("tests/samples/sample_1.gif").unwrap().read_to_end(&mut data).unwrap();
-        b.iter(|| {
-            let mut decoder = Decoder::new(&*data).read_info().unwrap();
-            let frame = decoder.read_next_frame().unwrap().unwrap();
-            test::black_box(frame);
-        });
-        let mut decoder = Decoder::new(&*data).read_info().unwrap();
-        b.bytes = decoder.read_next_frame().unwrap().unwrap().buffer.len() as u64
-    }
-    #[bench]
-    fn bench_big(b: &mut test::Bencher) {
-        let mut data = Vec::new();
-        File::open("tests/sample_big.gif").unwrap().read_to_end(&mut data).unwrap();
-        b.iter(|| {
-            let mut decoder = Decoder::new(&*data).read_info().unwrap();
-            let frame = decoder.read_next_frame().unwrap().unwrap();
-            test::black_box(frame);
-        });
-        let mut decoder = Decoder::new(&*data).read_info().unwrap();
-        b.bytes = decoder.read_next_frame().unwrap().unwrap().buffer.len() as u64
-    }*/
-    
     #[test]
     fn test_simple_indexed() {
-        let mut decoder = Decoder::new(File::open("tests/samples/sample_1.gif").unwrap()).read_info().unwrap();
+        let mut decoder = Decoder::new(File::open("tests/samples/sample_1.gif").unwrap()).unwrap();
         let frame = decoder.read_next_frame().unwrap().unwrap();
         assert_eq!(&*frame.buffer, &[
             1, 1, 1, 1, 1, 2, 2, 2, 2, 2,
@@ -476,92 +515,3 @@ mod test {
         }
     }
 }
-
-
-#[cfg(feature = "c_api")]
-mod c_interface {
-    use std::io::prelude::*;
-    use std::ptr;
-    use std::borrow::Cow;
-
-    use libc::c_int;
-    
-    use common::Block;
-
-    use c_api::{self, GifWord};
-    use c_api_utils::{CInterface, copy_colormap, copy_data, saved_images_new};
-
-    use super::decoder::{Decoded, DecodingError};
-
-    use super::{Reader};
-
-    impl<R> Reader<R> where R: Read + 'static {
-        /// Converts `Reader` into `CInterface`.
-        pub fn into_c_interface(self) -> Box<CInterface> {
-            Box::new(self)
-        }
-    }
-
-    impl<R: Read> CInterface for Reader<R> {
-        fn read_screen_desc(&mut self, this: &mut c_api::GifFileType) {
-            this.SWidth = self.width() as GifWord;
-            this.SHeight = self.height() as GifWord;
-            this.SColorResolution = 255;//self.global_palette().len() as GifWord;
-            this.SBackGroundColor = self.bg_color().unwrap_or(0) as GifWord;
-            this.AspectByte = 0;
-            self.offset = 0;
-        }
-
-        fn current_image_buffer(&mut self) -> Result<(&[u8], &mut usize), DecodingError> {
-            if let Cow::Borrowed(_) = self.current_frame.buffer {
-                self.read_next_frame()?;
-            }
-            Ok((&self.current_frame.buffer, &mut self.offset))
-        }
-
-        fn last_ext(&self) -> (u8, &[u8], bool) {
-            self.decoder.decoder.last_ext()
-        }
-
-        fn next_record_type(&mut self) -> Result<Block, DecodingError> {
-            loop {
-                match self.decoder.decode_next()? {
-                    Some(Decoded::BlockStart(type_)) => return Ok(type_),
-                    Some(_) => (),
-                    None => return Ok(Block::Trailer)
-                }
-            }
-        }
-    
-        fn decode_next(&mut self) -> Result<Option<Decoded>, DecodingError> {
-            self.decoder.decode_next()
-        }
-
-        /*
-        unsafe fn read_to_end(&mut self, this: &mut c_api::GifFileType) -> Result<(), DecodingError> {
-            self.read_screen_desc(this)?;
-            self.read_to_end()?;
-            this.ImageCount = self.frames().len() as c_int;
-            let images = saved_images_new(this.ImageCount as usize);
-            for (i, frame) in self.frames().iter().enumerate() {
-                *images.offset(i as isize) = c_api::SavedImage {
-                    ImageDesc: c_api::GifImageDesc {
-                        Left: frame.left as GifWord,
-                        Top: frame.top as GifWord,
-                        Width: frame.width as GifWord,
-                        Height: frame.height as GifWord,
-                        Interlace: num::FromPrimitive::from_u8(frame.interlaced as u8).unwrap(),
-                        ColorMap: copy_colormap(&frame.palette)
-                    },
-                    // on malloc(3) heap
-                    RasterBits: copy_data(&*frame.buffer),
-                    ExtensionBlockCount: 0,
-                    ExtensionBlocks: ptr::null_mut()
-                }
-                
-            }
-            this.SavedImages = images;
-            Ok(())
-        }*/
-    }
-}
